@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -46,6 +46,7 @@ class Serializer(object):
     parsed WAMP message objects and the bytes on wire (the transport).
     """
 
+    # WAMP defines the following 24 message types
     MESSAGE_TYPE_MAP = {
         message.Hello.MESSAGE_TYPE: message.Hello,
         message.Welcome.MESSAGE_TYPE: message.Welcome,
@@ -54,20 +55,16 @@ class Serializer(object):
         message.Authenticate.MESSAGE_TYPE: message.Authenticate,
         message.Goodbye.MESSAGE_TYPE: message.Goodbye,
         message.Error.MESSAGE_TYPE: message.Error,
-
         message.Publish.MESSAGE_TYPE: message.Publish,
         message.Published.MESSAGE_TYPE: message.Published,
-
         message.Subscribe.MESSAGE_TYPE: message.Subscribe,
         message.Subscribed.MESSAGE_TYPE: message.Subscribed,
         message.Unsubscribe.MESSAGE_TYPE: message.Unsubscribe,
         message.Unsubscribed.MESSAGE_TYPE: message.Unsubscribed,
         message.Event.MESSAGE_TYPE: message.Event,
-
         message.Call.MESSAGE_TYPE: message.Call,
         message.Cancel.MESSAGE_TYPE: message.Cancel,
         message.Result.MESSAGE_TYPE: message.Result,
-
         message.Register.MESSAGE_TYPE: message.Register,
         message.Registered.MESSAGE_TYPE: message.Registered,
         message.Unregister.MESSAGE_TYPE: message.Unregister,
@@ -120,7 +117,9 @@ class Serializer(object):
 
             message_type = raw_msg[0]
 
-            if type(message_type) != int:
+            if type(message_type) not in six.integer_types:
+                # CBOR doesn't roundtrip number types
+                # https://bitbucket.org/bodhisnarkva/cbor/issues/6/number-types-dont-roundtrip
                 raise ProtocolError("invalid type {0} for WAMP message type".format(type(message_type)))
 
             Klass = self.MESSAGE_TYPE_MAP.get(message_type)
@@ -136,76 +135,106 @@ class Serializer(object):
         return msgs
 
 
-#
 # JSON serialization is always supported
+
+import json
+import base64
+
+
+class _WAMPJsonEncoder(json.JSONEncoder):
+
+    def default(self, obj):
+        if isinstance(obj, six.binary_type):
+            return u'\x00' + base64.b64encode(obj).decode('ascii')
+        else:
+            return json.JSONEncoder.default(self, obj)
+
+
+#
+# the following is a hack. see http://bugs.python.org/issue29992
 #
-try:
-    # try import accelerated JSON implementation
-    #
-    import ujson
 
-    _json = ujson
+from json import scanner
+from json.decoder import scanstring
 
-    def _loads(val):
-        return ujson.loads(val, precise_float=True)
 
-    def _dumps(obj):
-        return ujson.dumps(obj, double_precision=15, ensure_ascii=False)
+def _parse_string(*args, **kwargs):
+    s, idx = scanstring(*args, **kwargs)
+    if s and s[0] == u'\x00':
+        s = base64.b64decode(s[1:])
+    return s, idx
 
-except ImportError:
-    # fallback to stdlib implementation
-    #
-    import json
 
-    _json = json
+class _WAMPJsonDecoder(json.JSONDecoder):
 
-    _loads = json.loads
+    def __init__(self, *args, **kwargs):
+        json.JSONDecoder.__init__(self, *args, **kwargs)
+        self.parse_string = _parse_string
 
-    def _dumps(obj):
-        return json.dumps(obj, separators=(',', ':'), ensure_ascii=False)
+        # we need to recreate the internal scan function ..
+        self.scan_once = scanner.py_make_scanner(self)
 
-finally:
-    class JsonObjectSerializer(object):
+        # .. and we have to explicitly use the Py version,
+        # not the C version, as the latter won't work
+        # self.scan_once = scanner.make_scanner(self)
 
-        JSON_MODULE = _json
+
+def _loads(s):
+    return json.loads(s, cls=_WAMPJsonDecoder)
+
+
+def _dumps(obj):
+    return json.dumps(obj,
+                      separators=(',', ':'),
+                      ensure_ascii=False,
+                      sort_keys=False,
+                      cls=_WAMPJsonEncoder)
+
+
+_json = json
+
+
+class JsonObjectSerializer(object):
+
+    JSON_MODULE = _json
+    """
+    The JSON module used (now only stdlib).
+    """
+
+    BINARY = False
+
+    def __init__(self, batched=False):
         """
-        The JSON module used (either stdib builtin or ujson).
+        Ctor.
+
+        :param batched: Flag that controls whether serializer operates in batched mode.
+        :type batched: bool
         """
+        self._batched = batched
 
-        BINARY = False
+    def serialize(self, obj):
+        """
+        Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
+        """
+        s = _dumps(obj)
+        if isinstance(s, six.text_type):
+            s = s.encode('utf8')
+        if self._batched:
+            return s + b'\30'
+        else:
+            return s
 
-        def __init__(self, batched=False):
-            """
-            Ctor.
-
-            :param batched: Flag that controls whether serializer operates in batched mode.
-            :type batched: bool
-            """
-            self._batched = batched
-
-        def serialize(self, obj):
-            """
-            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
-            """
-            s = _dumps(obj)
-            if isinstance(s, six.text_type):
-                s = s.encode('utf8')
-            if self._batched:
-                return s + b'\30'
-            else:
-                return s
-
-        def unserialize(self, payload):
-            """
-            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
-            """
-            if self._batched:
-                chunks = payload.split(b'\30')[:-1]
-            else:
-                chunks = [payload]
-            if len(chunks) == 0:
-                raise Exception("batch format error")
-            return [_loads(data.decode('utf8')) for data in chunks]
+    def unserialize(self, payload):
+        """
+        Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
+        """
+        if self._batched:
+            chunks = payload.split(b'\30')[:-1]
+        else:
+            chunks = [payload]
+        if len(chunks) == 0:
+            raise Exception("batch format error")
+        return [_loads(data.decode('utf8')) for data in chunks]
 
 
 IObjectSerializer.register(JsonObjectSerializer)
@@ -213,7 +242,7 @@ IObjectSerializer.register(JsonObjectSerializer)
 
 class JsonSerializer(Serializer):
 
-    SERIALIZER_ID = "json"
+    SERIALIZER_ID = u"json"
     """
     ID used as part of the WebSocket subprotocol name to identify the
     serializer with WAMP-over-WebSocket.
@@ -225,7 +254,7 @@ class JsonSerializer(Serializer):
     handshake identify the serializer with WAMP-over-RawSocket.
     """
 
-    MIME_TYPE = "application/json"
+    MIME_TYPE = u"application/json"
     """
     MIME type announced in HTTP request/response headers when running
     WAMP-over-Longpoll HTTP fallback.
@@ -240,17 +269,18 @@ class JsonSerializer(Serializer):
         """
         Serializer.__init__(self, JsonObjectSerializer(batched=batched))
         if batched:
-            self.SERIALIZER_ID = "json.batched"
+            self.SERIALIZER_ID = u"json.batched"
 
 
 ISerializer.register(JsonSerializer)
 
 
-#
-# MsgPack serialization depends on the `msgpack` package being available
+# MsgPack serialization depends on the `u-msgpack` package being available
+# https://pypi.python.org/pypi/u-msgpack-python
+# https://github.com/vsergeev/u-msgpack-python
 #
 try:
-    import msgpack
+    import umsgpack
 except ImportError:
     pass
 else:
@@ -262,15 +292,8 @@ else:
         Flag that indicates whether this serializer needs a binary clean transport.
         """
 
-        ENABLE_V5 = True
-        """
-        Enable version 5 of the MsgPack specification (which differentiates
-        between strings and binary).
-        """
-
         def __init__(self, batched=False):
             """
-            Ctor.
 
             :param batched: Flag that controls whether serializer operates in batched mode.
             :type batched: bool
@@ -281,7 +304,7 @@ else:
             """
             Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
             """
-            data = msgpack.packb(obj, use_bin_type=self.ENABLE_V5)
+            data = umsgpack.packb(obj)
             if self._batched:
                 return struct.pack("!L", len(data)) + data
             else:
@@ -292,22 +315,6 @@ else:
             Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
             """
 
-            def ensure_string_keys(d):
-                """
-                under python 2, with use_bin_type=True, most dict keys end up
-                getting encoded as bytes (any syntax except {u"key":
-                u"value"}) so instead of recursively looking through
-                everything that's getting serialized, we fix them up
-                on the way out using msgpack's `object_hook` as
-                there's no corresponding hook for serialization...
-                """
-                for (k, v) in six.iteritems(d):
-                    if not isinstance(k, six.text_type):
-                        newk = six.text_type(k, encoding='utf8')
-                        del d[k]
-                        d[newk] = v
-                return d
-
             if self._batched:
                 msgs = []
                 N = len(payload)
@@ -324,13 +331,7 @@ else:
                     data = payload[i + 4:i + 4 + l]
 
                     # append parsed raw message
-                    msgs.append(
-                        msgpack.unpackb(
-                            data,
-                            encoding='utf-8',
-                            object_hook=ensure_string_keys,
-                        )
-                    )
+                    msgs.append(umsgpack.unpackb(data))
 
                     # advance until everything consumed
                     i = i + 4 + l
@@ -340,11 +341,7 @@ else:
                 return msgs
 
             else:
-                unpacked = msgpack.unpackb(
-                    payload,
-                    encoding='utf-8',
-                    object_hook=ensure_string_keys,
-                )
+                unpacked = umsgpack.unpackb(payload)
                 return [unpacked]
 
     IObjectSerializer.register(MsgPackObjectSerializer)
@@ -353,7 +350,7 @@ else:
 
     class MsgPackSerializer(Serializer):
 
-        SERIALIZER_ID = "msgpack"
+        SERIALIZER_ID = u"msgpack"
         """
         ID used as part of the WebSocket subprotocol name to identify the
         serializer with WAMP-over-WebSocket.
@@ -365,7 +362,7 @@ else:
         handshake identify the serializer with WAMP-over-RawSocket.
         """
 
-        MIME_TYPE = "application/x-msgpack"
+        MIME_TYPE = u"application/x-msgpack"
         """
         MIME type announced in HTTP request/response headers when running
         WAMP-over-Longpoll HTTP fallback.
@@ -380,7 +377,7 @@ else:
             """
             Serializer.__init__(self, MsgPackObjectSerializer(batched=batched))
             if batched:
-                self.SERIALIZER_ID = "msgpack.batched"
+                self.SERIALIZER_ID = u"msgpack.batched"
 
     ISerializer.register(MsgPackSerializer)
 
@@ -463,7 +460,7 @@ else:
 
     class CBORSerializer(Serializer):
 
-        SERIALIZER_ID = "cbor"
+        SERIALIZER_ID = u"cbor"
         """
         ID used as part of the WebSocket subprotocol name to identify the
         serializer with WAMP-over-WebSocket.
@@ -475,7 +472,7 @@ else:
         handshake identify the serializer with WAMP-over-RawSocket.
         """
 
-        MIME_TYPE = "application/x-cbor"
+        MIME_TYPE = u"application/cbor"
         """
         MIME type announced in HTTP request/response headers when running
         WAMP-over-Longpoll HTTP fallback.
@@ -490,8 +487,116 @@ else:
             """
             Serializer.__init__(self, CBORObjectSerializer(batched=batched))
             if batched:
-                self.SERIALIZER_ID = "cbor.batched"
+                self.SERIALIZER_ID = u"cbor.batched"
 
     ISerializer.register(CBORSerializer)
 
     __all__.append('CBORSerializer')
+
+
+# UBJSON serialization depends on the `py-ubjson` package being available
+# https://pypi.python.org/pypi/py-ubjson
+# https://github.com/Iotic-Labs/py-ubjson
+try:
+    import ubjson
+except ImportError:
+    pass
+else:
+
+    class UBJSONObjectSerializer(object):
+
+        BINARY = True
+        """
+        Flag that indicates whether this serializer needs a binary clean transport.
+        """
+
+        def __init__(self, batched=False):
+            """
+
+            :param batched: Flag that controls whether serializer operates in batched mode.
+            :type batched: bool
+            """
+            self._batched = batched
+
+        def serialize(self, obj):
+            """
+            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
+            """
+            data = ubjson.dumpb(obj)
+            if self._batched:
+                return struct.pack("!L", len(data)) + data
+            else:
+                return data
+
+        def unserialize(self, payload):
+            """
+            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
+            """
+
+            if self._batched:
+                msgs = []
+                N = len(payload)
+                i = 0
+                while i < N:
+                    # read message length prefix
+                    if i + 4 > N:
+                        raise Exception("batch format error [1]")
+                    l = struct.unpack("!L", payload[i:i + 4])[0]
+
+                    # read message data
+                    if i + 4 + l > N:
+                        raise Exception("batch format error [2]")
+                    data = payload[i + 4:i + 4 + l]
+
+                    # append parsed raw message
+                    msgs.append(ubjson.loadb(data))
+
+                    # advance until everything consumed
+                    i = i + 4 + l
+
+                if i != N:
+                    raise Exception("batch format error [3]")
+                return msgs
+
+            else:
+                unpacked = ubjson.loadb(payload)
+                return [unpacked]
+
+    IObjectSerializer.register(UBJSONObjectSerializer)
+
+    __all__.append('UBJSONObjectSerializer')
+
+    class UBJSONSerializer(Serializer):
+
+        SERIALIZER_ID = u"ubjson"
+        """
+        ID used as part of the WebSocket subprotocol name to identify the
+        serializer with WAMP-over-WebSocket.
+        """
+
+        RAWSOCKET_SERIALIZER_ID = 4
+        """
+        ID used in lower four bits of second octet in RawSocket opening
+        handshake identify the serializer with WAMP-over-RawSocket.
+        """
+
+        MIME_TYPE = u"application/ubjson"
+        """
+        MIME type announced in HTTP request/response headers when running
+        WAMP-over-Longpoll HTTP fallback.
+        """
+
+        def __init__(self, batched=False):
+            """
+            Ctor.
+
+            :param batched: Flag to control whether to put this serialized into batched mode.
+            :type batched: bool
+            """
+            Serializer.__init__(self, UBJSONObjectSerializer(batched=batched))
+            if batched:
+                self.SERIALIZER_ID = u"ubjson.batched"
+
+    ISerializer.register(UBJSONSerializer)
+
+    __all__.append('UBJSONSerializer')
