@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,8 +32,8 @@ if os.environ.get('USE_TWISTED', False):
 
     from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
     from twisted.internet.defer import succeed, DeferredList
-    from twisted.python import log
     from twisted.trial import unittest
+    import twisted
     from six import PY3
 
     from autobahn import util
@@ -66,6 +66,11 @@ if os.environ.get('USE_TWISTED', False):
             self._handler.onMessage(msg)
             self._fake_router_session = ApplicationSession()
 
+        def drop_registration(self, reg_id):
+            self._handler.onMessage(
+                message.Unregistered(0, reg_id)
+            )
+
         def send(self, msg):
             if self._log:
                 payload, isbinary = self._serializer.serialize(msg)
@@ -79,6 +84,8 @@ if os.environ.get('USE_TWISTED', False):
                         reply = message.Published(msg.request, self._fake_router_session._request_id_gen.next())
                 elif len(msg.topic) == 0:
                     reply = message.Error(message.Publish.MESSAGE_TYPE, msg.request, u'wamp.error.invalid_uri')
+                elif msg.topic.startswith(u'noreply.'):
+                    pass
                 else:
                     reply = message.Error(message.Publish.MESSAGE_TYPE, msg.request, u'wamp.error.not_authorized')
 
@@ -188,6 +195,74 @@ if os.environ.get('USE_TWISTED', False):
             publication = yield handler.publish(u'com.myapp.topic1', 1, 2, 3, foo=23, bar='hello', options=types.PublishOptions(exclude_me=False, exclude=[100, 200, 300]))
             self.assertEqual(publication, None)
 
+            publication = yield handler.publish(u'com.myapp.topic1', 1, 2, 3, foo=23, bar='hello', options=types.PublishOptions(exclude_me=False, exclude=[100, 200, 300], retain=True))
+            self.assertEqual(publication, None)
+
+        @inlineCallbacks
+        def test_publish_outstanding_errors(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+
+            # this publish will "hang" because 'noreply.' URI is
+            # handled specially in MockTransport; so this request will
+            # be outstanding
+            publication = handler.publish(
+                u'noreply.foo',
+                options=types.PublishOptions(acknowledge=True),
+            )
+            # "leave" the session, which should trigger errbacks on
+            # all outstanding requests.
+            details = types.CloseDetails(reason=u'testing', message=u'how are you?')
+            yield handler.onLeave(details)
+
+            # ensure we got our errback
+            try:
+                yield publication
+            except ApplicationError as e:
+                self.assertEqual(u'testing', e.error)
+                self.assertEqual(u'how are you?', e.message)
+
+        @inlineCallbacks
+        def test_publish_outstanding_errors_async_errback(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            error_d = Deferred()
+
+            # this publish will "hang" because 'noreply.' URI is
+            # handled specially in MockTransport; so this request will
+            # be outstanding
+            publication_d = handler.publish(
+                u'noreply.foo',
+                options=types.PublishOptions(acknowledge=True),
+            )
+            # further, we add an errback that does some arbitrary async work
+            got_errors = []
+
+            def errback(fail):
+                got_errors.append(fail)
+                return error_d
+            publication_d.addErrback(errback)
+            # "leave" the session, which should trigger errbacks on
+            # all outstanding requests.
+            details = types.CloseDetails(reason=u'testing', message=u'how are you?')
+            handler.onLeave(details)
+
+            # since our errback is async, onLeave should not have
+            # completed yet but we should have already failed the
+            # publication
+            self.assertEqual(1, len(got_errors))
+            # ...now let the async errback continue by completing the
+            # Deferred we returned in our errback (could be fail or
+            # success, shoudln't matter)
+            error_d.callback(None)
+
+            # ensure we (now) get our errback
+            try:
+                yield publication_d
+            except ApplicationError as e:
+                self.assertEqual(u'testing', e.error)
+                self.assertEqual(u'how are you?', e.message)
+
         @inlineCallbacks
         def test_publish_acknowledged(self):
             handler = ApplicationSession()
@@ -280,6 +355,9 @@ if os.environ.get('USE_TWISTED', False):
             subscription = yield handler.subscribe(on_event, u'com.myapp.topic1', options=types.SubscribeOptions(match=u'wildcard'))
             self.assertTrue(type(subscription.id) in (int, long))
 
+            subscription = yield handler.subscribe(on_event, u'com.myapp.topic1', options=types.SubscribeOptions(match=u'wildcard', get_retained=True))
+            self.assertTrue(type(subscription.id) in (int, long))
+
         @inlineCallbacks
         def test_double_subscribe(self):
             handler = ApplicationSession()
@@ -463,37 +541,28 @@ if os.environ.get('USE_TWISTED', False):
             error_instance = RuntimeError("we have a problem")
             got_err_d = Deferred()
 
-            def observer(kw):
-                if kw['isError'] and 'failure' in kw:
-                    fail = kw['failure']
-                    fail.trap(RuntimeError)
-                    if error_instance == fail.value:
-                        got_err_d.callback(True)
-            log.addObserver(observer)
+            def observer(e, msg):
+                if error_instance == e.value:
+                    got_err_d.callback(True)
+            handler.onUserError = observer
 
             def boom():
                 raise error_instance
 
-            try:
-                sub = yield handler.subscribe(boom, u'com.myapp.topic1')
+            sub = yield handler.subscribe(boom, u'com.myapp.topic1')
 
-                # MockTransport gives us the ack reply and then we do our
-                # own event message
-                publish = yield handler.publish(
-                    u'com.myapp.topic1',
-                    options=types.PublishOptions(acknowledge=True, exclude_me=False),
-                )
-                msg = message.Event(sub.id, publish.id)
-                handler.onMessage(msg)
+            # MockTransport gives us the ack reply and then we do our
+            # own event message
+            publish = yield handler.publish(
+                u'com.myapp.topic1',
+                options=types.PublishOptions(acknowledge=True, exclude_me=False),
+            )
+            msg = message.Event(sub.id, publish.id)
+            handler.onMessage(msg)
 
-                # we know it worked if our observer worked and did
-                # .callback on our Deferred above.
-                self.assertTrue(got_err_d.called)
-                # ...otherwise trial will fail the test anyway
-                self.flushLoggedErrors()
-
-            finally:
-                log.removeObserver(observer)
+            # we know it worked if our observer worked and did
+            # .callback on our Deferred above.
+            self.assertTrue(got_err_d.called)
 
         @inlineCallbacks
         def test_unsubscribe(self):
@@ -531,6 +600,58 @@ if os.environ.get('USE_TWISTED', False):
             registration = yield handler.register(on_call, u'com.myapp.procedure1')
             yield registration.unregister()
 
+        def test_unregister_no_such_registration(self):
+            if twisted.version.major < 14:
+                return
+            handler = ApplicationSession()
+            transport = MockTransport(handler)
+
+            with self.assertRaises(ProtocolError) as ctx:
+                transport.send(
+                    message.Unregister(0, 1234)
+                )
+            self.assertIn(
+                "UNREGISTERED received for non-existant registration",
+                str(ctx.exception)
+            )
+
+        @inlineCallbacks
+        def test_unregister_log(self):
+            handler = ApplicationSession()
+            transport = MockTransport(handler)
+
+            def on_call(*args, **kwargs):
+                on_call.called = True
+            on_call.called = False
+
+            registration = yield handler.register(on_call, u'com.myapp.procedure1')
+            transport.drop_registration(registration.id)
+            self.assertFalse(on_call.called)
+
+        def test_on_disconnect_error(self):
+            errors = []
+
+            class AppSess(ApplicationSession):
+                def onUserError(self, e, msg):
+                    errors.append((e.value, msg))
+
+                def onDisconnect(self, foo, bar, quux='snark'):
+                    # this over-ridden onDisconnect takes the wrong args
+                    raise RuntimeError("This shouldn't happen")
+
+            s = AppSess()
+            MockTransport(s)
+
+            # when the transport closes, it calls onClose which should
+            # dispatch onDisconnect
+            s.onClose(False)
+
+            # ...but we should collect an error, because our
+            # onDisconnect takes the wrong arguments.
+            self.assertEqual(len(errors), 1)
+            self.assertTrue(isinstance(errors[0][0], TypeError))
+
+    class TestInvoker(unittest.TestCase):
         @inlineCallbacks
         def test_invoke(self):
             handler = ApplicationSession()
@@ -573,9 +694,9 @@ if os.environ.get('USE_TWISTED', False):
             # (the register) and then increment for the call()
             def verify_seq_id(orig, msg):
                 if isinstance(msg, message.Register):
-                    self.assertEqual(msg.request, 0)
-                elif isinstance(msg, message.Call):
                     self.assertEqual(msg.request, 1)
+                elif isinstance(msg, message.Call):
+                    self.assertEqual(msg.request, 2)
                 return orig(msg)
             orig0 = trans0.send
             orig1 = trans1.send
@@ -598,6 +719,11 @@ if os.environ.get('USE_TWISTED', False):
             handler = ApplicationSession()
             handler.traceback_app = True
             MockTransport(handler)
+            errors = []
+
+            def log_error(e, msg):
+                errors.append((e.value, msg))
+            handler.onUserError = log_error
 
             name_error = NameError('foo')
 
@@ -618,9 +744,8 @@ if os.environ.get('USE_TWISTED', False):
 
             # also, we should have logged the real NameError to
             # Twisted.
-            errs = self.flushLoggedErrors()
-            self.assertEqual(1, len(errs))
-            self.assertEqual(name_error, errs[0].value)
+            self.assertEqual(1, len(errors))
+            self.assertEqual(name_error, errors[0][0])
 
         @inlineCallbacks
         def test_invoke_progressive_result(self):
@@ -674,6 +799,11 @@ if os.environ.get('USE_TWISTED', False):
 
             got_progress = Deferred()
             progress_error = NameError('foo')
+            logged_errors = []
+
+            def got_error(e, msg):
+                logged_errors.append((e.value, msg))
+            handler.onUserError = got_error
 
             def progress(arg, something=None):
                 self.assertEqual('nothing', something)
@@ -693,15 +823,15 @@ if os.environ.get('USE_TWISTED', False):
                 options=types.CallOptions(on_progress=progress),
                 key='word',
             )
+
             self.assertEqual(42, res)
             # our progress handler raised an error, but not before
             # recording success.
             self.assertTrue(got_progress.called)
             self.assertEqual('life', got_progress.result)
             # make sure our progress-handler error was logged
-            errs = self.flushLoggedErrors()
-            self.assertEqual(1, len(errs))
-            self.assertEqual(progress_error, errs[0].value)
+            self.assertEqual(1, len(logged_errors))
+            self.assertEqual(progress_error, logged_errors[0][0])
 
         @inlineCallbacks
         def test_invoke_progressive_result_no_args(self):
@@ -768,6 +898,43 @@ if os.environ.get('USE_TWISTED', False):
             self.assertTrue(got_progress.called)
             self.assertEqual('word', got_progress.result)
 
+        @inlineCallbacks
+        def test_call_exception_runtimeerror(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            exception = RuntimeError("a simple error")
+
+            def raiser():
+                raise exception
+
+            registration0 = yield handler.register(raiser, u"com.myapp.myproc_error")
+            try:
+                yield handler.call(u'com.myapp.myproc_error')
+                self.fail()
+            except Exception as e:
+                self.assertIsInstance(e, ApplicationError)
+                self.assertEqual(e.error_message(), "wamp.error.runtime_error: a simple error")
+            finally:
+                yield registration0.unregister()
+
+        @inlineCallbacks
+        def test_call_exception_bare(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            exception = Exception()
+
+            def raiser():
+                raise exception
+
+            registration0 = yield handler.register(raiser, u"com.myapp.myproc_error")
+            try:
+                yield handler.call(u'com.myapp.myproc_error')
+                self.fail()
+            except Exception as e:
+                self.assertIsInstance(e, ApplicationError)
+            finally:
+                yield registration0.unregister()
+
         # ## variant 1: works
         # def test_publish1(self):
         #    d = self.handler.publish(u'de.myapp.topic1')
